@@ -12,7 +12,9 @@ patched_modules = defaultdict(lambda: [])
 
 from baseten.baseten_deployed_model import BasetenDeployedModel  # noqa: E402
 from baseten.baseten_deployed_model import install_requirements, requirements_status  # noqa: E402, F401
-from baseten.common.model_deployer import deploy_custom_model, deploy_model_object  # noqa: E402
+from baseten.common.model_deployer import deploy_custom_model, deploy_model_object, \
+    build_and_deploy_baseten_scaffold, build_and_deploy_baseten_custom_scaffold, build_baseten_scaffold, \
+    build_baseten_custom_scaffold, deploy_baseten_scaffold  # noqa: E402
 from baseten.common.util import setup_logger  # noqa: E402
 from baseten.common import settings  # noqa: E402
 
@@ -25,8 +27,16 @@ logger.debug(f'Starting the client with the server URL set to {settings.API_URL_
 
 # The Baseten model ID that either a) the user initialized baseten with, or b) was created when deploying a model.
 working_model_id = None
-deploy = deploy_model_object  # This allows the user to call baseten.deploy(model)
-deploy_custom = deploy_custom_model  # This allows the user to call baseten.deploy_custom(...)
+
+deploy = build_and_deploy_baseten_scaffold  # This allows the user to call baseten.deploy(model)
+deploy_custom = build_and_deploy_baseten_custom_scaffold  # This allows the user to call baseten.deploy_custom(...)
+
+build_scaffold = build_baseten_scaffold
+build_custom_scaffold = build_baseten_custom_scaffold
+deploy_scaffold = deploy_baseten_scaffold
+
+deploy_legacy = deploy_model_object  # This allows the user to call baseten.deploy_legacy(model)
+deploy_custom_legacy = deploy_custom_model  # This allows the user to call baseten.deploy_custom_legacy(...)
 
 
 def set_log_level(level):
