@@ -41,6 +41,79 @@ def models(api_key):
     return resp['data']
 
 
+@with_api_key
+def create_model_from_scaffold(
+    api_key,
+    model_name,
+    s3_key,
+    model_framework,
+    model_type,
+    model_framework_req,
+    dockerfile,
+    build_args,
+    semver_bump
+):
+    encoded_model_framework_req = base64_encoded_json_str(model_framework_req)
+    encoded_build_args = base64_encoded_json_str(build_args)
+    query_string = f'''
+    mutation {{
+      create_model_from_scaffold(name: "{model_name}"
+                   s3_key: "{s3_key}",
+                   model_framework: "{model_framework}",
+                   model_type: "{model_type}",
+                   semver_bump: "{semver_bump}",
+                   encoded_model_framework_req: "{encoded_model_framework_req}",
+                   dockerfile: "{dockerfile}"
+                   encoded_build_args: "{encoded_build_args}"
+      ) {{
+        id,
+        name,
+        version_id
+      }}
+    }}
+    '''
+
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']['create_model_from_scaffold']
+
+
+@with_api_key
+def create_model_version_from_scaffold(
+    api_key,
+    model_id,
+    s3_key,
+    model_framework,
+    model_type,
+    model_framework_req,
+    dockerfile,
+    build_args,
+    semver_bump
+):
+    encoded_model_framework_req = base64_encoded_json_str(model_framework_req)
+    encoded_build_args = base64_encoded_json_str(build_args)
+    query_string = f'''
+    mutation {{
+      create_model_version_from_scaffold(
+                   model_id: "{model_id}"
+                   s3_key: "{s3_key}",
+                   model_framework: "{model_framework}",
+                   model_type: "{model_type}",
+                   semver_bump: "{semver_bump}",
+                   encoded_model_framework_req: "{encoded_model_framework_req}",
+                   dockerfile: "{dockerfile}"
+                   encoded_build_args: "{encoded_build_args}"
+      ) {{
+        id,
+        name,
+        version_id
+      }}
+    }}
+    '''
+
+    resp = _post_graphql_query(api_key, query_string)
+    return resp['data']['create_model_version_from_scaffold']
+
+
 @with_api_key
 def create_model(api_key,
                  model_name,
@@ -88,7 +161,7 @@ def create_model(api_key,
     return resp['data']['create_model']
 
 
-@ with_api_key
+@with_api_key
 def models_versions(api_key, model_id):
     query_string = f'''
     {{
@@ -107,7 +180,7 @@ def models_versions(api_key, model_id):
     return resp['data']['model']
 
 
-@ with_api_key
+@with_api_key
 def model_version_input_shape(api_key, model_version_id):
     query_string = f'''
     {{
@@ -121,7 +194,7 @@ def model_version_input_shape(api_key, model_version_id):
     return resp['data']['model_version']['input_shape']
 
 
-@ with_api_key
+@with_api_key
 def create_model_version(api_key,
                          model_id,
                          s3_key,
@@ -165,7 +238,7 @@ def create_model_version(api_key,
     return resp['data']['create_model_version']
 
 
-@ with_api_key
+@with_api_key
 def signed_s3_upload_post(api_key, model_file_name):
     query_string = f'''
     {{
@@ -185,7 +258,7 @@ def signed_s3_upload_post(api_key, model_file_name):
     return resp['data']['signed_s3_upload_url']
 
 
-@ with_api_key
+@with_api_key
 def register_data_for_model(api_key, s3_key, model_version_id, data_name):
     query_string = f'''
     mutation {{
