@@ -5,6 +5,7 @@ import json
 import logging
 import pathlib
 import re
+import tarfile
 import tempfile
 import zipfile
 from functools import reduce
@@ -13,7 +14,15 @@ from typing import IO, Any, Callable, Dict, List, Optional, Tuple
 
 import joblib
 import requests
+from baseten_scaffolding.definitions.base import \
+    AbstractModelScaffoldDefinition
+from baseten_scaffolding.model_inference import (infer_pytorch_packages,
+                                                 infer_sklearn_packages,
+                                                 infer_tensorflow_packages)
+from baseten_scaffolding.scaffold.build import scaffold, scaffold_custom
 from colorama import Fore, Style
+from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
+from tqdm import tqdm
 
 import baseten
 from baseten.baseten_deployed_model import BasetenDeployedModel
@@ -23,12 +32,9 @@ from baseten.common.core import (FrameworkNotSupportedError,
                                  ModelFilesMissingError, ModelInputShapeError,
                                  ModelNotSupportedError, Semver,
                                  raises_api_error)
-from baseten.common.settings import (API_URL_BASE, CUSTOM, PYTORCH, SKLEARN,
-                                     TENSORFLOW)
-from baseten.common.util import (infer_pytorch_packages,
-                                 infer_sklearn_packages,
-                                 infer_tensorflow_packages,
-                                 parse_requirements_file, zipdir)
+from baseten.common.settings import (API_URL_BASE, CUSTOM, KERAS, PYTORCH,
+                                     SKLEARN, TENSORFLOW)
+from baseten.common.util import parse_requirements_file, zipdir
 
 logger = logging.getLogger(__name__)
 
@@ -79,15 +85,14 @@ def deploy_model(serialized_model: IO[Any],
     if not model_framework_req:
         model_framework_req = {}
 
-    logger.info('Making contact with BaseTen 👋 👽')
+    logger.info('👽 👋 Making contact with BaseTen 👋 👽')
 
-    logger.info('Uploading model.')
     file_ext = _model_ext_from_framework(model_framework)
     model_key = _upload_model(serialized_model, file_ext)
     model_info = _exists_model(model_name, baseten.working_model_id, api.models)
     if not model_info:
         logger.debug('No model found with provided name/id.')
-        logger.info(f'Creating new model with name {model_name}')
+        logger.debug(f'Creating new model named {Fore.BLUE}{model_name}{Style.RESET_ALL}')
         model_and_version_json = api.create_model(model_name,
                                                   model_key,
                                                   model_framework,
@@ -99,8 +104,8 @@ def deploy_model(serialized_model: IO[Any],
                                                   model_framework_req,
                                                   version_bump,
                                                   model_files_dict)
-        logger.info(f'Created model:\n{json.dumps(model_and_version_json, indent=4)}')
-        logger.info(f'Successfully registered model {Fore.BLUE}{model_name}{Style.RESET_ALL}.')
+        logger.debug(f'Created model:\n{json.dumps(model_and_version_json, indent=4)}')
+        logger.info(f'Successfully created new model named {Fore.BLUE}{model_name}{Style.RESET_ALL}.')
 
         model_id = model_and_version_json['id']
         model_version_id = model_and_version_json['version_id']
@@ -123,11 +128,11 @@ def deploy_model(serialized_model: IO[Any],
 
         model_version_id = model_version_json['id']
         logger.info(
-            f'Successfully created version {Fore.BLUE}{model_version_id}{Style.RESET_ALL} for{model_name}.')
+            f'Successfully created version {Fore.BLUE}{model_version_id}{Style.RESET_ALL} for {model_name}.')
 
-    logger.info(f'{Fore.BLUE}Deploying model version.')
+    logger.debug(f'{Fore.BLUE}Deploying model version.')
     model_version_web_url = f'{API_URL_BASE}/models/{model_id}/versions/{model_version_id}'
-    logger.info('🏁 The model is being deployed right now 🏁')
+    logger.info('🏁 Deploying model 🏁')
     visit_message = f'|  Visit {Fore.BLUE}{model_version_web_url}{Style.RESET_ALL} for deployment status  |'
     visit_message_len = len(visit_message) - len(Fore.BLUE) - len(Style.RESET_ALL)
     logger.info(''.join(['-' for _ in range(visit_message_len)]))
@@ -228,6 +233,14 @@ def deploy_custom_model(
     return deployed_model
 
 
+def _legacy_model_framework(model_framework: str):
+    # tf.Keras does not always resolve to tensorflow, in newer versions of TF (>2.3.4) it will show as keras
+    # in the legacy world we still want this to be tensorflow
+    if model_framework == KERAS:
+        return TENSORFLOW
+    return model_framework
+
+
 @raises_api_error
 def deploy_model_object(
     model: Any,
@@ -262,9 +275,11 @@ def deploy_model_object(
     if hasattr(model, 'baseten'):
         model = copy.deepcopy(model)
         del model.baseten
-    logger.info('🤖 Extracting 🤖 model 🤖 metadata 🤖')
+    logger.info('🤖 Extracting model metadata 🤖')
     model_class = model.__class__
     model_framework, _, _ = model_class.__module__.partition('.')
+    model_framework = _legacy_model_framework(model_framework)
+
     if model_framework not in {SKLEARN, TENSORFLOW}:
         try:
             import torch
@@ -330,15 +345,257 @@ def deploy_model_object(
                                       model_files_dict=model_files_dict)
         serialized_model.close()
 
-    if deployed_model:
-        deployed_model.generate_model_template(
-            model_type,
-            model_framework,
-            feature_names
-        )
     return deployed_model
 
 
+def build_baseten_scaffold(
+    model: Any,
+    model_name: str = None,
+    model_files: List[str] = None,
+    data_files: List[str] = None,
+    target_directory: str = None,
+    requirements_file: str = None,
+):
+    """Create a scaffold directory with the given model. A scaffold directory is a build context designed to be built
+    as a container locally or uploaded into a baseten serving environment.
+
+    Args:
+        model (an in-memory model object): A model object to be deployed (e.g. a keras, sklearn, or pytorch model
+            object)
+        model_name (str, optional): The name of the model to be created, if necessary.
+        model_files (List[str], optional): Additional files required for model definition, e.g class
+            definitions for pytorch models or other python code. Can be a glob that resolves to files for the root
+            directory or a directory path. Files and directories here will be on the Python path
+        data_files (List[str], optional): Additional files required for model operation. Can be a glob that resolves to
+            files for the root directory or a directory path.
+        target_directory (str, optional): The local directory target for the scaffold. Otherwise a temporary directory
+            will be generated
+        requirements_file (str, optional): A file of packages in a PIP requirements format to be installed in the
+            container environment.
+    Returns:
+        AbstractModelScaffoldDefinition
+    """
+    if not model_name:
+        import coolname
+        model_name = coolname.generate_slug(2)
+    local_name = model_name.replace(' ', '_')
+
+    b10_scaffold = scaffold(
+        model=model,
+        model_files=model_files,
+        data_files=data_files,
+        target_directory=target_directory,
+        requirements_file=requirements_file,
+    )
+    logger.info(f'To build this model server locally execute '
+                f'`{b10_scaffold.docker_build_string} -t {local_name.lower()}`')
+    logger.info(f'To run this model server locally execute `docker run --rm -p 8080:8080 {local_name.lower()}`')
+    logger.info(f'To use the Python shell locally execute `docker run --rm -it {local_name.lower()} python3`')
+    return b10_scaffold
+
+
+def build_baseten_custom_scaffold(
+    model_name: str,
+    model_files: List[str] = None,
+    data_files: List[str] = None,
+    target_directory: str = None,
+    requirements_file: str = None,
+    model_class: str = None,
+):
+    """
+    Create a scaffold directory for a custom model. A scaffold directory is a build context designed to be built as a
+    container locally or uploaded into a baseten serving environment.
+
+        Args:
+            model_name (str, optional): The name of the model to be created, if necessary.
+            model_files (List[str], optional): Additional files required for model definition, e.g class
+                definitions for pytorch models or other python code. Can be a glob that resolves to files for the root
+                directory or a directory path. Files and directories here will be on the Python path
+            data_files (List[str], optional): Additional files required for model operation. Can be a glob that resolves
+                to files for the root directory or a directory path.
+            target_directory (str, optional): The local directory target for the scaffold. Otherwise a temporary
+                directory will be generated
+            requirements_file (str, optional): A file of packages in a PIP requirements format to be installed in the
+                container environment.
+            model_class (str, optional): The python class name for a model object that exists as a code within
+                `model_files`
+        Returns:
+            AbstractModelScaffoldDefinition
+    """
+    if not model_name:
+        import coolname
+        model_name = coolname.generate_slug(2)
+    local_name = model_name.replace(' ', '_')
+
+    b10_scaffold = scaffold_custom(
+        model_files=model_files,
+        data_files=data_files,
+        target_directory=target_directory,
+        requirements_file=requirements_file,
+        model_class=model_class
+    )
+    logger.info(f'To build this model server locally execute '
+                f'`{b10_scaffold.docker_build_string} -t {local_name.lower()}`')
+    logger.info(f'To run this model server locally execute `docker run --rm -p 8080:8080 {local_name.lower()}`')
+    logger.info(f'To use the Python shell locally execute `docker run --rm -it {local_name.lower()} python3`')
+    logger.info('Be sure to read the `README.md` for guidance on construction of a working custom scaffold')
+    return b10_scaffold
+
+
+@raises_api_error
+def deploy_baseten_scaffold(
+    b10_scaffold: AbstractModelScaffoldDefinition,
+    model_name: str = None,
+    version_bump: Semver = Semver.MINOR.value,
+):
+    """
+    Given an existing baseten scaffold object, deploy it onto the baseten infastructure.
+
+    Args:
+        b10_scaffold: A scaffold object representing a local baseten_scaffold directory
+        model_name (str, optional): The name of the model to be created, if necessary.
+        version_bump (str, optional): The version bump for this deployment, one of 'MAJOR', 'MINOR', 'PATCH'.
+
+    Returns:
+        BasetenDeployedModel
+    """
+    logger.info(f'Serializing {Fore.BLUE}{model_name}{Style.RESET_ALL} scaffold.')
+    temp_file = tempfile.NamedTemporaryFile(suffix='.tgz')
+    with tarfile.open(temp_file.name, 'w:gz') as tar:
+        tar.add(b10_scaffold.scaffold_dir, arcname='.')
+    temp_file.file.seek(0)
+
+    model_info = _exists_model(model_name, baseten.working_model_id, api.models)
+
+    logger.info('Making contact with BaseTen 👋 👽')
+    s3_key = _upload_model(temp_file, 'tgz')
+    if not model_info:
+        model_version_json = api.create_model_from_scaffold(
+            model_name,
+            s3_key,
+            b10_scaffold.model_framework,
+            b10_scaffold.model_type,
+            b10_scaffold.model_framework_requirements,
+            b10_scaffold.dockerfile,
+            b10_scaffold.build_args,
+            version_bump
+        )
+        logger.info(f'Created model:\n{json.dumps(model_version_json, indent=4)}')
+        logger.info(f'Successfully registered model {Fore.BLUE}{model_name}{Style.RESET_ALL}.')
+
+        model_id = model_version_json['id']
+        model_version_id = model_version_json['version_id']
+        # Set the newly created model model_version_json be the working model for future commands
+        baseten.working_model_id = model_id
+    else:
+        model_id, model_name = model_info
+        logger.info(f"Model '{model_name}' found. Creating a new version.")
+
+        model_version_json = api.create_model_version_from_scaffold(model_id,
+                                                                    s3_key,
+                                                                    b10_scaffold.model_framework,
+                                                                    b10_scaffold.model_type,
+                                                                    b10_scaffold.model_framework_requirements,
+                                                                    b10_scaffold.dockerfile,
+                                                                    b10_scaffold.build_args,
+                                                                    version_bump)
+
+        model_version_id = model_version_json['id']
+        logger.info(
+            f'Successfully created version {Fore.BLUE}{model_version_id}{Style.RESET_ALL} for {model_name}.')
+
+    logger.info(f'{Fore.BLUE}Deploying model version.')
+    model_version_web_url = f'{API_URL_BASE}/models/{model_id}/versions/{model_version_id}'
+    logger.info('🏁 The model is being built and deployed right now 🏁')
+    visit_message = f'|  Visit {Fore.BLUE}{model_version_web_url}{Style.RESET_ALL} for deployment status  |'
+    visit_message_len = len(visit_message) - len(Fore.BLUE) - len(Style.RESET_ALL)
+    logger.info(''.join(['-' for _ in range(visit_message_len)]))
+    logger.info(visit_message)
+    logger.info(''.join(['-' for _ in range(visit_message_len)]))
+    return BasetenDeployedModel(model_version_id=model_version_id, model_name=model_name)
+
+
+def build_and_deploy_baseten_scaffold(
+    model: Any,
+    model_name: str = None,
+    model_files: List[str] = None,
+    data_files: List[str] = None,
+    target_directory: str = None,
+    requirements_file: str = None,
+    version_bump: Semver = Semver.MINOR.value,
+):
+    """Create a scaffold directory with the given model and upload it to the basten serving environment. A scaffold
+    directory is a build context designed to be built as a container locally or uploaded into a baseten serving
+    environment.
+
+    Args:
+        model (an in-memory model object): A model object to be deployed (e.g. a keras, sklearn, or pytorch model
+            object)
+        model_name (str, optional): The name of the model to be created, if necessary.
+        model_files (List[str], optional): Additional files required for model definition, e.g class
+            definitions for pytorch models or other python code. Can be a glob that resolves to files for the root
+            directory or a directory path. Files and directories here will be on the Python path
+        data_files (List[str], optional): Additional files required for model operation. Can be a glob that resolves to
+            files for the root directory or a directory path.
+        target_directory (str, optional): The local directory target for the scaffold. Otherwise a temporary directory
+            will be generated
+        requirements_file (str, optional): A file of packages in a PIP requirements format to be installed in the
+            container environment.
+        version_bump (str, optional): The version bump for this deployment, one of 'MAJOR', 'MINOR', 'PATCH'.
+    Returns:
+        BasetenDeployedModel
+    """
+
+    b10_scaffold = build_baseten_scaffold(
+        model, model_name, model_files, data_files, target_directory, requirements_file
+    )
+    return deploy_baseten_scaffold(
+        b10_scaffold,
+        model_name,
+        version_bump
+    )
+
+
+def build_and_deploy_baseten_custom_scaffold(
+    model_name: str = None,
+    model_files: List[str] = None,
+    data_files: List[str] = None,
+    target_directory: str = None,
+    requirements_file: str = None,
+    model_class: str = None,
+    version_bump: Semver = Semver.MINOR.value,
+):
+    """
+    Create a scaffold directory for a custom model and upload it into the baseten serving environment.
+
+        Args:
+            model_name (str, optional): The name of the model to be created, if necessary.
+            model_files (List[str], optional): Additional files required for model definition, e.g class
+                definitions for pytorch models or other python code. Can be a glob that resolves to files for the root
+                directory or a directory path. Files and directories here will be on the Python path
+            data_files (List[str], optional): Additional files required for model operation. Can be a glob that resolves
+                to files for the root directory or a directory path.
+            target_directory (str, optional): The local directory target for the scaffold. Otherwise a temporary
+                directory will be generated
+            requirements_file (str, optional): A file of packages in a PIP requirements format to be installed in the
+                container environment.
+            model_class (str, optional): The python class name for a model object that exists as a code within
+                `model_files`
+            version_bump (str, optional): The version bump for this deployment, one of 'MAJOR', 'MINOR', 'PATCH'.
+    Returns:
+        BasetenDeployedModel
+    """
+
+    b10_scaffold = build_baseten_custom_scaffold(
+        model_name, model_files, data_files, target_directory, requirements_file, model_class
+    )
+    return deploy_baseten_scaffold(
+        b10_scaffold,
+        model_name,
+        version_bump
+    )
+
+
 def _upload_model(serialized_model: IO, file_ext: str) -> str:
     """Uploads the serialized model to the appropriate environment
 
@@ -358,13 +615,25 @@ def _upload_model(serialized_model: IO, file_ext: str) -> str:
     logger.debug(f'Signed s3 upload post:\n{json.dumps(signed_s3_upload_post, indent=4)}')
 
     # Upload the serialized model to S3
-    files = {'file': (model_file_name, serialized_model)}
     form_fields = signed_s3_upload_post['form_fields']
     form_fields['AWSAccessKeyId'] = form_fields.pop('aws_access_key_id')  # S3 expects key name AWSAccessKeyId
+    form_fields['file'] = (model_file_name, serialized_model)
     logger.info('🚀 Uploading model to BaseTen 🚀')
-    resp = requests.post(signed_s3_upload_post['url'], data=form_fields, files=files)
+    encoder = MultipartEncoder(fields=form_fields)
+    encoder_len = encoder.len
+    pbar = tqdm(total=encoder_len, unit_scale=True,
+                bar_format="Upload Progress: {percentage:3.0f}%% |%s{bar:100}%s| {n_fmt}/{total_fmt}"
+                % (Fore.BLUE, Fore.RESET))
+
+    def callback(monitor):
+        progress = monitor.bytes_read - pbar.n
+        pbar.update(progress)
+
+    monitor = MultipartEncoderMonitor(encoder, callback)
+    resp = requests.post(signed_s3_upload_post['url'], data=monitor, headers={'Content-Type': monitor.content_type})
     resp.raise_for_status()
-    logger.info('🖖 Upload success! 🖖')
+    pbar.close()
+    logger.info('🔮 Upload successful!🔮')
 
     logger.debug(f'File upload HTTP status code: {resp.status_code} and content:\n{resp.content}')
 
